@@ -1,10 +1,25 @@
 use itertools::Itertools;
 use lazy_static::lazy_static;
 use regex::Regex;
+use std::str::FromStr;
 use unicode_truncate::UnicodeTruncateStr;
 
 mod sigfig;
 
+/// Represents the type of a value.
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+pub enum ValueType {
+    Boolean,
+    Integer,
+    Double,
+    Date,
+    Time,
+    DateTime,
+    Character,
+    /// A missing value.
+    Na,
+}
+
 pub fn is_logical(text: &str) -> bool {
     // col_logical -l, T,F,TRUE,FALSE,True,False,true,false,t,f,1,0
     lazy_static! {
@@ -24,13 +39,7 @@ pub fn is_integer(text: &str) -> bool {
 }
 
 pub fn is_double(text: &str) -> bool {
-    lazy_static! {
-        // for exp values, but seems to match other strings also
-        //static ref R: Regex = Regex::new(r"[+-]?[0-9]+(\.[0-9]+)?([Ee][+-]?[0-9]+)?").unwrap();
-        static ref R: Regex = Regex::new(r"^[+-]?([0-9]+([.][0-9]*)?|[.][0-9]+)$").unwrap();
-
-    }
-    R.is_match(text)
+    f64::from_str(text).is_ok()
 }
 
 pub fn is_time(text: &str) -> bool {
@@ -82,21 +91,23 @@ pub fn is_na_string_padded(text: &str) -> bool {
 
 // utilities
 
-pub fn infer_type_from_string(text: &str) -> &'static str {
+pub fn infer_type_from_string(text: &str) -> ValueType {
     if is_time(text) {
-        "<tst>"
+        ValueType::Time
     } else if is_logical(text) {
-        "<lgl>"
+        ValueType::Boolean
     } else if is_integer(text) {
-        "<int>"
+        ValueType::Integer
     } else if is_date_time(text) {
-        "<tdt>"
+        ValueType::DateTime
     } else if is_date(text) {
-        "<tsd>"
+        ValueType::Date
     } else if is_double(text) {
-        "<dbl>"
+        ValueType::Double
+    } else if text.is_empty() | is_na(text) {
+        ValueType::Na
     } else {
-        "<chr>"
+        ValueType::Character
     }
 }
 
@@ -126,7 +137,8 @@ pub fn trunc_strings(vec_col: &[&str], width: usize) -> Vec<String> {
 pub fn header_len_str(vec_col: &[&str]) -> Vec<usize> {
     vec_col
         .iter()
-        .map(|&string| string.chars().count())
+        .map(|&string| format_if_num(&string))
+        .map(|string| string.chars().count())
         .collect::<Vec<usize>>()
 }
 
@@ -141,7 +153,7 @@ pub fn format_if_na(text: &str) -> String {
     string.to_string()
 }
 
-fn format_if_num(text: &str) -> String {
+pub fn format_if_num(text: &str) -> String {
     if is_double(text) {
         let xf = text.to_string().parse::<f64>().unwrap();
         let x = sigfig::DecimalSplits { val: xf, sigfig: 3 };
@@ -153,6 +165,7 @@ fn format_if_num(text: &str) -> String {
             rhs: x.rhs(),
             dec: x.dec(),
             final_string: x.final_string(),
+            rhs_string_len: x.rhs_string_len(x.final_string()),
             sigfig_index_lhs_or_rhs: x.sigfig_index_lhs_or_rhs(),
             sigfig_index_from: x.sigfig_index_from(),
             sigfig_index_to: x.sigfig_index_to(),
@@ -163,11 +176,12 @@ fn format_if_num(text: &str) -> String {
     }
 }
 
-pub fn get_col_data_type(col: &[&str]) -> &'static str {
+pub fn get_col_data_type(col: &[&str]) -> ValueType {
     // counts the frequency of the datatypes in the column
-    // returns the most frequent. todo-make na not count and handle ties
+    // returns the most frequent while ignoring NA values.
     col.iter()
         .map(|x| infer_type_from_string(x))
+        .filter(|x| !matches!(x, &ValueType::Na))
         .group_by(|&x| x)
         .into_iter()
         .map(|(key, group)| (key, group.count()))
