@@ -1,6 +1,8 @@
-use csv::ReaderBuilder;
+use csv::{Reader, ReaderBuilder};
 use owo_colors::OwoColorize;
-use std::io::{self};
+use std::fs::File;
+use std::io::{self, BufReader, Read};
+use std::path::PathBuf;
 use structopt::StructOpt;
 mod datatype;
 use crossterm::terminal::size;
@@ -77,6 +79,9 @@ struct Cli {
         help = "Print object details to make it easier for the maintainer to find and resolve bugs."
     )]
     debug_mode: bool,
+
+    #[structopt(name = "FILE", parse(from_os_str), help = "File to process")]
+    file: Option<PathBuf>,
 }
 
 fn main() {
@@ -85,8 +90,8 @@ fn main() {
     //println!("rows {} cols {}", term_tuple.1, term_tuple.0);
     let opt = Cli::from_args();
     let color_option = opt.color;
-    let title_option = opt.title;
-    let footer_option = opt.footer;
+    let title_option = &opt.title;
+    let footer_option = &opt.footer;
     let row_display_option = opt.row_display;
     // nord
     let nord_meta_color = (143, 188, 187);
@@ -157,10 +162,22 @@ fn main() {
     };
 
     //   colname reader
-    let mut r = ReaderBuilder::new()
-        .has_headers(false)
-        .delimiter(opt.delimiter)
-        .from_reader(io::stdin());
+    let reader_result = build_reader(&opt);
+    let mut r = if let Ok(reader) = reader_result {
+        reader
+    } else {
+        // We can safetly use unwrap, becouse if file in case when file is None
+        // build_reader would return readeer created from stdin
+        let path_buf = opt.file.unwrap();
+        let path = path_buf.as_path();
+        if let Some(path) = path.to_str() {
+            eprintln!("Failed to open file: {}", path);
+        } else {
+            eprintln!("Failed to open file.")
+        }
+        return;
+    };
+
     let rdr = r
         .records()
         .into_iter()
@@ -191,11 +208,16 @@ fn main() {
         v.push(column)
     }
 
+    if debug_mode {
+        println!("{:?}", "v");
+        println!("{:?}", v);
+    }
+
     if debug_mode {
         // make datatypes vector
-        let mut vec_datatypes: Vec<&str> = vec!["#"; cols as usize];
-        for i in 0..cols {
-            vec_datatypes[i] = datatype::get_col_data_type(&v[i]);
+        let mut vec_datatypes = Vec::with_capacity(cols);
+        for column in &v {
+            vec_datatypes.push(datatype::get_col_data_type(&column))
         }
         println!("{:?}", "vec_datatypes");
         println!("{:?}", vec_datatypes);
@@ -282,7 +304,7 @@ fn main() {
         (cols).truecolor(meta_color.0, meta_color.1, meta_color.2),
     );
     // title
-    if !datatype::is_na(&title_option) {
+    if !datatype::is_na(title_option) {
         print!("{: <6}", "");
         println!(
             "{}",
@@ -380,7 +402,7 @@ fn main() {
     }
 
     // footer
-    if !datatype::is_na(&footer_option) {
+    if !datatype::is_na(footer_option) {
         println!("{: <6}", "");
         println!(
             "{}",
@@ -391,6 +413,22 @@ fn main() {
     println!();
 } // end main
 
+fn build_reader(opt: &Cli) -> Result<Reader<Box<dyn Read>>, std::io::Error> {
+    let source: Box<dyn Read> = if let Some(path) = opt.file.clone() {
+        let file = File::open(path)?;
+        Box::new(BufReader::new(file))
+    } else {
+        Box::new(io::stdin())
+    };
+
+    let reader = ReaderBuilder::new()
+        .has_headers(false)
+        .delimiter(opt.delimiter)
+        .from_reader(source);
+
+    Ok(reader)
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -417,4 +455,187 @@ mod tests {
         assert_eq!(datatype::is_na(&"1".to_string()), false);
         assert_eq!(datatype::is_na(&"0".to_string()), false);
     }
+    // the following tests look messy, but the formatting is a neccesary condition.
+    #[test]
+    fn a_csv() {
+        let v: Vec<Vec<&str>> = vec![
+            vec![
+                "name",
+                "abc",
+                "abc",
+                "abc",
+                "abc",
+                "abc",
+                "abcde",
+                "abcdefgh",
+                "abcdefghijkl",
+                "NA",
+                "abcdefghijklmnop",
+                "",
+                "n/a",
+                "floatlike",
+            ],
+            vec![
+                "value",
+                "0.00000001",
+                "0.0000001",
+                "0.000001",
+                "0.00001",
+                "0.0001",
+                "0.001",
+                "0.01",
+                "0.1",
+                "1",
+                "10",
+                "100",
+                "",
+                "2/ 2.5 Gallon",
+            ],
+            vec![
+                "bool", "T", "T", "T", "T", "T", "F", "True", "F", "T", "F", "", "F", "F",
+            ],
+            vec![
+                "date",
+                "2021-01-01",
+                "2021-01-01",
+                "2021-01-01",
+                "2021-01-01",
+                "2021-01-01",
+                "2021-01-01",
+                "",
+                "2021-01-01",
+                "2021-01-01",
+                "2021-01-01",
+                "2021-01-01",
+                "2021-01-01",
+                "2021-01-01",
+            ],
+        ];
+        let col_largest_width_post_proc: Vec<usize> = vec![16, 13, 4, 10];
+        let mut vf: Vec<Vec<String>> = vec![vec!["#".to_string(); 13 as usize]; 4 as usize];
+        for i in 0..col_largest_width_post_proc.len() {
+            vf[i] = datatype::trunc_strings(&v[i], col_largest_width_post_proc[i]);
+        }
+
+        assert_eq!(
+            vf,
+            [
+                [
+                    "name             ",
+                    "abc              ",
+                    "abc              ",
+                    "abc              ",
+                    "abc              ",
+                    "abc              ",
+                    "abcde            ",
+                    "abcdefgh         ",
+                    "abcdefghijkl     ",
+                    "NA               ",
+                    "abcdefghijklmnop ",
+                    "NA               ",
+                    "NA               ",
+                    "floatlike        "
+                ],
+                [
+                    "value         ",
+                    "0.00000001    ",
+                    "0.0000001     ",
+                    "0.000001      ",
+                    "0.00001       ",
+                    "0.0001        ",
+                    "0.001         ",
+                    "0.01          ",
+                    "0.1           ",
+                    "1             ",
+                    "10            ",
+                    "100           ",
+                    "NA            ",
+                    "2/ 2.5 Gallon "
+                ],
+                [
+                    "bool ", "T    ", "T    ", "T    ", "T    ", "T    ", "F    ", "True ",
+                    "F    ", "T    ", "F    ", "NA   ", "F    ", "F    "
+                ],
+                [
+                    "date       ",
+                    "2021-01-01 ",
+                    "2021-01-01 ",
+                    "2021-01-01 ",
+                    "2021-01-01 ",
+                    "2021-01-01 ",
+                    "2021-01-01 ",
+                    "NA         ",
+                    "2021-01-01 ",
+                    "2021-01-01 ",
+                    "2021-01-01 ",
+                    "2021-01-01 ",
+                    "2021-01-01 ",
+                    "2021-01-01 "
+                ]
+            ]
+        );
+    }
+
+    #[test]
+    fn long_doubles_74_csv() {
+        let v: Vec<Vec<&str>> = vec![
+            vec!["text", "row1", "row2"],
+            vec!["col1", "3.333333333333333", "1.11111111111111111"],
+            vec!["col2", "3.333333333333333", "1.11111111111111111"],
+            vec!["col3", "3.333333333333333", "1.11111111111111111"],
+        ];
+        let col_largest_width_post_proc: Vec<usize> = vec![4, 4, 4, 4];
+        let mut vf: Vec<Vec<String>> = vec![vec!["#".to_string(); 3 as usize]; 4 as usize];
+        for i in 0..col_largest_width_post_proc.len() {
+            vf[i] = datatype::trunc_strings(&v[i], col_largest_width_post_proc[i]);
+        }
+
+        assert_eq!(
+            vf,
+            [
+                ["text ", "row1 ", "row2 "],
+                ["col1 ", "3.33 ", "1.11 "],
+                ["col2 ", "3.33 ", "1.11 "],
+                ["col3 ", "3.33 ", "1.11 "]
+            ]
+        );
+    }
+
+    #[test]
+    fn unicode_pr55_csv() {
+        let v: Vec<Vec<&str>> = vec![
+            vec!["aColumn", "1"],
+            vec!["bColumn", "üÜğĞçÇşŞöÖ"],
+            vec!["cColumn", "üÜğĞçÇşŞöÖ üÜğĞçÇşŞöÖ"],
+            vec!["dColumn", "77"],
+            vec!["eColumn", "TR"],
+            vec!["fColumn", "77"],
+            vec!["gColumn", "77"],
+        ];
+        let col_largest_width_post_proc: Vec<usize> = vec![7, 10, 20, 7, 7, 7, 7];
+        let mut vf: Vec<Vec<String>> = vec![vec!["#".to_string(); 2 as usize]; 7 as usize];
+        for i in 0..col_largest_width_post_proc.len() {
+            vf[i] = datatype::trunc_strings(&v[i], col_largest_width_post_proc[i]);
+        }
+
+        assert_eq!(
+            vf,
+            [
+                ["aColumn ", "1       "],
+                ["bColumn    ", "üÜğĞçÇşŞöÖ "],
+                ["cColumn              ", "üÜğĞçÇşŞöÖ üÜğĞçÇşŞ… "],
+                ["dColumn ", "77      "],
+                ["eColumn ", "TR      "],
+                ["fColumn ", "77      "],
+                ["gColumn ", "77      "]
+            ]
+        );
+    }
+
+    #[test]
+    fn build_reader_can_create_reader_without_file_specified() {
+        let cli = Cli::from_args();
+        let reader = build_reader(&cli);
+        assert!(reader.is_ok());
+    }
 }
